/* 
 *  =========================================================================
 *  TestChainOfResponsibilityPattern.java: Form chain as:
 * 
 *  --- IssueRaiser -> FaxErrorhandler -> EmailErrorHandler ...
 * 
 *  Note: Messages are processed based on keywords in message content.
 * 
 *  Modified by: Mark Austin                                      March, 2019 
 *  =========================================================================
 */ 

package chain01;

public class TestChainOfResponsibilityPattern {
   public static void main(String[] args) {
      Receiver faxHandler, emailHandler;            

      System.out.printf("Test Chain of Responsibility Pattern ...\n");
      System.out.printf("==================================== ...\n");

      // Create objects that will be in chain ...

      IssueRaiser issueRaiser = new IssueRaiser();		
      faxHandler   = new FaxErrorHandler();
      emailHandler = new EmailErrorHandler();

      // =====================================
      // Assemble chain of responsibility ....
      // =====================================

      issueRaiser.setFirstErrorHandler(faxHandler);

      // FaxErrorHandler will pass the error to EmailHandler if needed. 

      faxHandler.nextErrorHandler(emailHandler);

      // EmailErrorHandler will be placed at the last position in the chain

      emailHandler.nextErrorHandler(null);	

      // Create list of messages ...

      Message m1 = new Message("Fax is going slow.", MessagePriority.NORMAL);
      Message m2 = new Message("Emails are not reaching.", MessagePriority.HIGH);
      Message m3 = new Message("In Email, CC field is disabled always.", MessagePriority.NORMAL);
      Message m4 = new Message("Fax is not reaching destinations.", MessagePriority.HIGH);

      // Pass messages to issueRaiser ...

      issueRaiser.raiseMessage(m1);
      issueRaiser.raiseMessage(m2);
      issueRaiser.raiseMessage(m3);
      issueRaiser.raiseMessage(m4);

      System.out.printf("==================================== ...\n");
   }
}
