/**
 * ==============================================================
 * MainViewJavaFX.java: Create non-modular JavaFX View .....
 * ==============================================================
 */

package mvc03.view;

import javafx.application.Platform;

import javafx.scene.Scene;
import javafx.stage.Stage;
import javafx.scene.image.*;
import javafx.scene.input.*;
import javafx.scene.layout.*;
import javafx.scene.*;
import javafx.scene.Scene;
import javafx.scene.control.*;
import javafx.scene.layout.VBox;
import javafx.scene.paint.Color;
import javafx.scene.shape.*;
import javafx.scene.text.Font;
import javafx.scene.text.Text;
import javafx.scene.transform.Rotate;
import javafx.scene.chart.NumberAxis;
import javafx.collections.FXCollections;
import javafx.collections.ObservableList;
import javafx.embed.swing.JFXPanel;
import javafx.event.ActionEvent;
import javafx.event.EventHandler;
import javafx.geometry.HPos;
import javafx.geometry.Insets;
import javafx.geometry.VPos;
import javafx.geometry.Side;

import java.awt.BorderLayout;
import java.awt.Dimension;

import javafx.application.Platform;

import javafx.beans.InvalidationListener;
import javafx.beans.Observable;
import javafx.beans.value.*;
import javafx.beans.value.ChangeListener;
import javafx.beans.value.ObservableValue;
import javafx.beans.property.ObjectProperty;
import javafx.beans.property.SimpleObjectProperty;
import javafx.beans.property.BooleanProperty;
import javafx.beans.property.SimpleBooleanProperty;
import javafx.collections.FXCollections;
import javafx.collections.ObservableList;
import javafx.embed.swing.JFXPanel;

import javafx.event.*;
import javafx.geometry.*;

import javax.swing.*;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.SwingUtilities;

import mvc03.controller.MainController;
import mvc03.model.MainModel;

public class MainViewJavaFX {
   private final MainModel model;
   private final MainController controller;

   private static JFXPanel chartFxPanel;
   private static JFrame frame;
   private static int xSize = 700;
   private static int ySize = 700;

   private double minX   =  -00;
   private double maxX   =  500;
   private double minY   =  -00;
   private double maxY   =  500;
   private int iGridSize =  100;
   final static Group group = new Group();

   NumberAxis xAxis;
   NumberAxis yAxis;

   // Referencs to basic shapes and grid

   private SVGPath axes;

   // Constructor methods ...

   public MainViewJavaFX(MainModel model, MainController controller) {
        this.model      = model;
        this.controller = controller;
   }

   public void setSize ( int xSize, int ySize ) {
      this.xSize = xSize;
      this.ySize = ySize;
   }

   // ==================================================================
   // Viewer2D setup ....
   // ==================================================================

   public static void main ( String args[] ) {}

   public void init() {
      System.out.printf("--- Enter MainViewJavaFX.init() ... \n");

      // Put javafx panel inside a JPanel ...

      chartFxPanel = new JFXPanel();
      chartFxPanel.setPreferredSize( new Dimension( xSize, ySize ) );
      chartFxPanel.setLayout(new BorderLayout());

      // Create JavaFX scene

      Platform.runLater(new Runnable() {
         @Override
         public void run() {
            createScene();
         }
      });

      System.out.printf("--- Leave MainViewJavaFX.init() ... \n");
   }

   // =========================================================
   // Create scene ...
   // =========================================================

   public void createScene() {
      System.out.printf("--- Enter MainViewJavaFX.createScene() ... \n");

      xAxis = new NumberAxis( minX, maxX, iGridSize );
      yAxis = new NumberAxis( minY, maxY, iGridSize );

      // Layout for x- and y- axes ....

      xAxis.setSide(Side.BOTTOM);
      xAxis.setAutoRanging(false);
      xAxis.setScaleX( 1 );
      xAxis.setScaleY(-1 );
      xAxis.setTranslateX( minX );
      xAxis.setTranslateY( (minY - 50) );
      xAxis.setMinWidth( (double) ( maxX - minX ) );
      xAxis.setLabel( "x axis (m)" );
      xAxis.setStyle( "-fx-font: NORMAL 16 arial;" );

      yAxis.setSide(Side.LEFT);
      yAxis.setAutoRanging(false);
      yAxis.setScaleX( 1 );
      yAxis.setScaleY(-1 );
      yAxis.setTranslateY( minY );
      yAxis.setTranslateX( (minX-75) );
      yAxis.setMinHeight( (double) ( maxY - minY ) );
      yAxis.setLabel( "y axis (m)" );
      yAxis.setStyle( "-fx-font: NORMAL 16 arial;" );

      // Create axes ...

      axes = createAxes();

      // Create circle ...

      CirclePane circleView = new CirclePane( model.getCircleModel(), controller.getCircleController() );

      createListeners();

      // Assemble group layout ...

      group.getChildren().add( xAxis );
      group.getChildren().add( yAxis );
      group.getChildren().add( axes );
      group.getChildren().add(circleView);

      group.setScaleX(    group.getScaleX() );
      group.setScaleY(    group.getScaleY() );
      group.setScaleY( -1*group.getScaleY() );

      // Assemble window layout ...

      ScrollPane zoomPane = createZoomPane(group);

      VBox layout = new VBox();
      layout.getChildren().add( zoomPane );

      VBox.setVgrow(zoomPane, Priority.ALWAYS );

      Scene scene = new Scene(layout, xSize, ySize );

      // scene.getStylesheets().add("resources/engviewer.css");

      chartFxPanel.setScene( scene );

      System.out.printf("--- Leave MainViewJavaFX.createScene() ... \n");
   }

   // =========================================================
   // Zoomable interface support ....
   // =========================================================
  
   private ScrollPane createZoomPane(final Group group) {
     final double SCALE_DELTA = 1.1;

     final StackPane zoomPane = new StackPane();
     zoomPane.getChildren().add(group);

     final ScrollPane scroller = new ScrollPane();
     final Group scrollContent = new Group(zoomPane);
     scroller.setContent(scrollContent);

     scroller.viewportBoundsProperty().addListener(new ChangeListener<Bounds>() {
        @Override
        public void changed(ObservableValue<? extends Bounds> observable, Bounds oldValue, Bounds newValue) {
               zoomPane.setMinSize(newValue.getWidth(), newValue.getHeight());
           }
        });
    
     scroller.setPrefViewportWidth(256);
     scroller.setPrefViewportHeight(256);
     scroller.setVvalue( 1.0 );

     zoomPane.setOnScroll( new EventHandler<ScrollEvent>() {
        @Override
        public void handle(ScrollEvent event) {
           event.consume();

           if (event.getDeltaY() == 0) {
              return;
           }

           double scaleFactor = (event.getDeltaY() > 0) ? SCALE_DELTA : 1 / SCALE_DELTA;

           // amount of scrolling in each direction in scrollContent coordinate units

           Point2D scrollOffset = figureScrollOffset(scrollContent, scroller);

           group.setScaleX(group.getScaleX() * scaleFactor);
           group.setScaleY(group.getScaleY() * scaleFactor);

           // move viewport so that old center remains in the 
           // center after the scaling

           repositionScroller(scrollContent, scroller, scaleFactor, scrollOffset);
        }
    } );

    // ===================================================================
    // Panning via drag....
    // ===================================================================

    final ObjectProperty<Point2D> lastMouseCoordinates = new SimpleObjectProperty<Point2D>();

    scrollContent.setOnMousePressed(new EventHandler<MouseEvent>() {
      @Override
      public void handle(MouseEvent event) {
         lastMouseCoordinates.set(new Point2D(event.getX(), event.getY()));
      }
    });

    scrollContent.setOnMouseDragged(new EventHandler<MouseEvent>() {
      @Override
      public void handle(MouseEvent event) {
        double deltaX     = event.getX() - lastMouseCoordinates.get().getX();
        double extraWidth = scrollContent.getLayoutBounds().getWidth() - scroller.getViewportBounds().getWidth();
        double deltaH     = deltaX * (scroller.getHmax() - scroller.getHmin()) / extraWidth;
        double desiredH   = scroller.getHvalue() - deltaH;
        scroller.setHvalue(Math.max(0, Math.min(scroller.getHmax(), desiredH)));

        double deltaY      = event.getY() - lastMouseCoordinates.get().getY();
        double extraHeight = scrollContent.getLayoutBounds().getHeight() - scroller.getViewportBounds().getHeight();
        double deltaV   = deltaY * (scroller.getHmax() - scroller.getHmin()) / extraHeight;
        double desiredV = scroller.getVvalue() - deltaV;
        scroller.setVvalue(Math.max(0, Math.min(scroller.getVmax(), desiredV)));
      }
    });
 
     return scroller;
   }

   private Point2D figureScrollOffset(Node scrollContent, ScrollPane scroller) {
      double extraWidth        = scrollContent.getLayoutBounds().getWidth() - scroller.getViewportBounds().getWidth();
      double hScrollProportion = (scroller.getHvalue() - scroller.getHmin()) / (scroller.getHmax() - scroller.getHmin());
      double scrollXOffset = hScrollProportion * Math.max(0, extraWidth);
      double extraHeight   = scrollContent.getLayoutBounds().getHeight() - scroller.getViewportBounds().getHeight();
      double vScrollProportion = (scroller.getVvalue() - scroller.getVmin()) / (scroller.getVmax() - scroller.getVmin());
      double scrollYOffset     = vScrollProportion * Math.max(0, extraHeight);
      return new Point2D(scrollXOffset, scrollYOffset);
   }

   private void repositionScroller(Node scrollContent, ScrollPane scroller, double scaleFactor, Point2D scrollOffset) {
      double scrollXOffset = scrollOffset.getX();
      double scrollYOffset = scrollOffset.getY();

      double extraWidth = scrollContent.getLayoutBounds().getWidth() - scroller.getViewportBounds().getWidth();

      if (extraWidth > 0) {
        double halfWidth = scroller.getViewportBounds().getWidth() / 2 ;
        double newScrollXOffset = (scaleFactor - 1) *  halfWidth + scaleFactor * scrollXOffset;
        scroller.setHvalue(scroller.getHmin() + newScrollXOffset * (scroller.getHmax() - scroller.getHmin()) / extraWidth);
      } else {
        scroller.setHvalue(scroller.getHmin());
      }

      double extraHeight = scrollContent.getLayoutBounds().getHeight() - scroller.getViewportBounds().getHeight();

      if (extraHeight > 0) {
         double halfHeight = scroller.getViewportBounds().getHeight() / 2 ;
         double newScrollYOffset = (scaleFactor - 1) * halfHeight + scaleFactor * scrollYOffset;
         scroller.setVvalue(scroller.getVmin() + newScrollYOffset * (scroller.getVmax() - scroller.getVmin()) / extraHeight);
      } else {
         scroller.setHvalue(scroller.getHmin());
      }
   }

   // ================================================================
   // First, try to position axes at corrdinate (0,0). Otherwise,
   // position axes in bottom left-hand corner.
   // ================================================================

   private SVGPath createAxes() {
      SVGPath axes = new SVGPath();

      if ( minX <= 0 && minY <= 0 ) {
         axes.setContent("M100,00 L0,0 L0,100");
      } else {
         StringBuffer buf = new StringBuffer();
         buf.append("M");
         buf.append( String.format("%d,%d", (int) minX, (int) (minY + 100) ) );
         buf.append(" ");
         buf.append( String.format("%d,%d", (int) minX, (int) minY ) );
         buf.append(" ");
         buf.append( String.format("%d,%d", (int) (minX + 100), (int) minY ) );
         axes.setContent( buf.toString() );
      }

      axes.setStroke(Color.RED);
      axes.setStrokeWidth(2);
      axes.setFill(null);

      return axes;
   }

   // =========================================================
   // Display the 2D view ....
   // =========================================================

   public void display() {
        System.out.printf("--- Enter MainViewJavaFX.display() ... \n");
        SwingUtilities.invokeLater(new Runnable() {

            @Override
            public void run() {
                System.out.printf("--- Starting MainViewJavaFX.run() ... \n");
                
                init();

                frame = new JFrame("MVC Circle Application");
                frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
                frame.setLocationRelativeTo(null);
                frame.add( chartFxPanel );
                frame.setVisible(true);
                frame.setSize( xSize, ySize );
                frame.pack();
            }
        });
   }

   // =========================================================
   // Create listeners ...
   // =========================================================

    private void createListeners() {
        group.setOnMouseMoved( e -> controller.getCircleController().mousePosChanged(e) );
        group.setOnMouseClicked( e -> controller.getCircleController().mouseClicked(e) );
    }
}
