/*
 *  ====================================================================
 *  Wheel.java: Specification for a car wheel ....
 *
 *  Note: The method:
 *
 *     Wheel.accept( CarElementVisitor )
 *
 *  overrides CarElement.accept(CarElementVisitor), so the call to
 *  accept is bound at run time. This can be considered the first
 *  dispatch. The decision to call:
 * 
 *     CarElementVisitor.visit(Wheel)
 *
 *  however, rather than the other 'visit' methods in CarElementVisitor,
 *  can be made during compile time since 'this' is known at compile
 *  time to be a Wheel. Moreover, each subclass of CarElementVisitor
 *  overrides the visit(Wheel), which is another decision that is
 *  made at run time. This can be considered the second dispatch.
 *
 *  Modified by: Mark Austin                                  June, 2012
 *  ====================================================================
 */

package visitor01;
 
public class Wheel implements CarElement {
   private String  name;
   private double price;

   // Constructor methods ....
 
   public Wheel(String name) {
      this.name = name;
   }

   // Set/get methods for body name and price ....

   public String getName() {
      return this.name;
   }

   public void setPrice ( double price ) {
      this.price = price;
   }

   public double getPrice () {
      return price;
   }

   // Accept method for CarElement visitor ...
 
   public void accept( CarElementVisitor visitor ) {
      visitor.visit( this );
   }
}
