/** 
  *  ===============================================================
  *  Point.java: Create two-dimensional (x,y) homogeneous point.
  *
  *  Parameters
  *
  *  -- protected double   x, y;
  *  -- protected double radius;
  *
  *  Constructor methods
  *
  *  -- public Point()
  *  -- public Point( double x, double y );
  *
  *  Written by: Mark Austin                          November, 2004
  *  ===============================================================
  */ 

package visitor02.model.primitive;

import visitor02.model.*;

public class Point extends AbstractFeature {
    protected double radius;

    // Point constructor methods .....

    public Point() {
        super();
        setX( 0.0 );
        setY( 0.0 );
        radius = 0.0;
    }

    public Point(String sName, double x, double y ) {
        super();
        setName( sName );
        setX( x );
        setY( y );
        radius = 0.0;
    }

    public Point(double x, double y, double radius ) {
        super();
        setX( x );
        setY( y );
        this.radius = radius;
    }

    // Retrieve point radius ...

    public double getRadius() {
       return this.radius;
    }

    // Compute norm of the point ....
 
    public double norm() {
        return Math.sqrt( getX() * getX() + getY() * getY() );
    }

    // Clone (i.e., deep copy) point object ...

    public Point clone() {
       try {
           Point cloned = (Point) super.clone();
           return cloned;
       } catch ( CloneNotSupportedException e) {
           System.out.println("*** Error in Point.clone() ... ");
           return null;
       }
    }

    // Create string representation of a point ...
    
    public String toString() {
       return "Point(" + getName() + "): (x,y) = (" + getX() + ", " + getY() + ")";
    }
}
