/*
 *  ===================================================================
 *  ShoppingCart.java: Simple shopping cart model ... 
 *  ===================================================================
 */

package visitor04;

import java.util.ArrayList;
import java.util.List;

public class ShoppingCart {
   private ArrayList<Visitable> items;

   // Constructor method ....

   public ShoppingCart() {
      items = new ArrayList<Visitable>();
   }

   // Add item to the shopping cart ...

   public void add( Visitable item ) {
      items.add ( item );
   }

   // Retrieve no items in shopping cart ...

   public int getNoItemsInCart() {
      return items.size();
   }

   // ---------------------------------------------------
   // Print details of sales offer ...
   // ---------------------------------------------------

   public String calculateSalesOffer() {

      // Create a sales offer visitor, then iterate through items ...

      SalesOfferVisitor visitor = new SalesOfferVisitor();
      for(Visitable item: items) {
          item.accept( visitor );
      }

      return visitor.toString();
   }

   // ---------------------------------------------------
   // Calculate total cost of items in shopping cart ....
   // ---------------------------------------------------

   public double calculateTotalCost() {

      // Create billing visitor, then iterate through items ...

      BillingVisitor visitor = new BillingVisitor();
      for(Visitable item: items) {
          item.accept(visitor);
      }

      // Return total price of items in shopping cart ...

      double price = visitor.getTotalPrice();
      return price;
   }
}
